/*
 * Taken from:
 * https://stackoverflow.com/questions/7360473/linux-non-blocking-fifo-on-demand-logging
 */
/* ftee - clone stdin to stdout and to a named pipe 
(c) racic@stackoverflow
WTFPL Licence */

char *taken_from="Source code for ftee.c: https://stackoverflow.com/questions/7360473/linux-non-blocking-fifo-on-demand-logging";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
  int readfd, writefd;
  struct stat status;
  char *fifonam;
  char buffer[BUFSIZ];
  ssize_t bytes;

  signal(SIGPIPE, SIG_IGN);

  if(argc < 2)
    {
      printf("Usage:\n someprog 2>&1 | %s FIFO PID\n FIFO - path to a"
	     " named pipe, required argument\n PID - pidfile\n", argv[0]);
      exit(EXIT_FAILURE);
    }
  fifonam = argv[1];

  /* save the pid to argv[2] if specified */
  if(argc == 3) {
    FILE *fp;
    if((fp = fopen(argv[2], "w")) == (FILE*) 0) {
      printf("Error: could not open [%s] for write.\n", argv[2]);
      perror("fopen");
      exit(1);
    }
    fprintf(fp, "%d", getpid());
    fclose(fp);
  }

  readfd = open(fifonam, O_RDONLY | O_NONBLOCK);
  if(-1==readfd)
    {
      perror("ftee: readfd: open()");
      exit(EXIT_FAILURE);
    }

  if(-1==fstat(readfd, &status))
    {
      perror("ftee: fstat");
      close(readfd);
      exit(EXIT_FAILURE);
    }

  if(!S_ISFIFO(status.st_mode))
    {
      printf("ftee: %s in not a fifo!\n", fifonam);
      close(readfd);
      exit(EXIT_FAILURE);
    }

  writefd = open(fifonam, O_WRONLY | O_NONBLOCK);
  if(-1==writefd)
    {
      perror("ftee: writefd: open()");
      close(readfd);
      exit(EXIT_FAILURE);
    }

  close(readfd);

  while(1)
    {
      bytes = read(STDIN_FILENO, buffer, sizeof(buffer));
      if (bytes < 0 && errno == EINTR)
	continue;
      if (bytes <= 0)
	break;

      bytes = write(STDOUT_FILENO, buffer, bytes);
      if(-1==bytes)
	perror("ftee: writing to stdout");
      bytes = write(writefd, buffer, bytes);
      if(-1==bytes);//Ignoring the errors
    }
  close(writefd); 
  return(0);
}
